from collective.solr import manager
from collective.solr.solr import SolrConnection


# TODO: Remove whole file in next major release.


class FtwSolrConnection(SolrConnection):
    pass


class SolrConnectionManager(manager.SolrConnectionManager):
    pass
